==> yass-proxy-debug-1.11.4-1-x86_64.pkg.tar.zst: usr/src/debug/yass-proxy/yass-1.11.4/src/net/dot_request.hpp <==
// SPDX-License-Identifier: GPL-2.0
/* Copyright (c) 2024 Chilledheart  */

#ifndef H_NET_DOT_REQUEST_HPP
#define H_NET_DOT_REQUEST_HPP

#include <absl/strings/str_format.h>

#include "core/ref_counted.hpp"
#include "core/scoped_refptr.hpp"
#include "net/asio.hpp"
#include "net/dns_message.hpp"
#include "net/network.hpp"
#include "net/ssl_socket.hpp"

namespace net {

class DoTRequest : public RefCountedThreadSafe<DoTRequest> {
 public:
  DoTRequest(int ssl_socket_data_index,
             asio::io_context& io_context,
             asio::ip::tcp::endpoint endpoint,
             const std::string& dot_host,
             int dot_port,
             SSL_CTX* ssl_ctx)
      : io_context_(io_context),
        socket_(io_context),
        endpoint_(endpoint),
        dot_host_(dot_host),
        dot_port_(dot_port),
        ssl_socket_(nullptr),
        ssl_socket_data_index_(ssl_socket_data_index),
        ssl_ctx_(ssl_ctx) {}

  template <typename... Args>
  static scoped_refptr<DoTRequest> Create(Args&&... args) {
    return MakeRefCounted<DoTRequest>(std::forward<Args>(args)...);
  }
  ~DoTRequest();

  void close();

  using AsyncResolveCallback = absl::AnyInvocable<void(asio::error_code ec, struct addrinfo* addrinfo)>;
  void DoRequest(dns_message::DNStype dns_type, const std::string& host, int port, AsyncResolveCallback cb);

 private:
  // tcp connect
  // ssl handshake
  // generate HTTP request
  // send HTTP request
  // receive HTTP response
  // parse HTTP body

  void OnSocketConnect();
  void OnSSLConnect();
  void OnSSLWritable(asio::error_code ec);
  void OnSSLReadable(asio::error_code ec);
  void OnReadHeader();
  void OnReadBody();
  void OnParseDnsResponse();
  void OnDoneRequest(asio::error_code ec, struct addrinfo* addrinfo);

 private:
  enum ReadState {
    Read_Header,
    Read_Body,
  };
  ReadState read_state_ = Read_Header;
  size_t body_length_ = 0;

 private:
  asio::io_context& io_context_;
  asio::ip::tcp::socket socket_;
  asio::ip::tcp::endpoint endpoint_;

  const std::string& dot_host_;
  const int dot_port_;
  scoped_refptr<SSLSocket> ssl_socket_;
  const int ssl_socket_data_index_;
  SSL_CTX* ssl_ctx_;

  bool closed_ = false;
  dns_message::DNStype dns_type_;
  std::string host_;
  int port_;
  std::string service_;
  AsyncResolveCallback cb_;
  std::shared_ptr<IOBuf> buf_;
  std::shared_ptr<IOBuf> recv_buf_;
};

}  // namespace net

#endif  // H_NET_DOT_REQUEST_HPP

==> yass-proxy-debug-1.16.1-1-x86_64.pkg.tar.zst: usr/src/debug/yass-proxy/yass-1.16.1/src/net/dot_request.hpp <==
// SPDX-License-Identifier: GPL-2.0
/* Copyright (c) 2024 Chilledheart  */

#ifndef H_NET_DOT_REQUEST_HPP
#define H_NET_DOT_REQUEST_HPP

#include <absl/strings/str_format.h>
#include <base/memory/ref_counted.h>
#include <base/memory/scoped_refptr.h>

#include "net/asio.hpp"
#include "net/dns_message.hpp"
#include "net/network.hpp"
#include "net/ssl_socket.hpp"

namespace net {

class DoTRequest : public gurl_base::RefCountedThreadSafe<DoTRequest> {
 public:
  DoTRequest(int ssl_socket_data_index,
             asio::io_context& io_context,
             asio::ip::tcp::endpoint endpoint,
             const std::string& dot_host,
             int dot_port,
             SSL_CTX* ssl_ctx)
      : io_context_(io_context),
        socket_(io_context),
        endpoint_(endpoint),
        dot_host_(dot_host),
        dot_port_(dot_port),
        ssl_socket_(nullptr),
        ssl_socket_data_index_(ssl_socket_data_index),
        ssl_ctx_(ssl_ctx) {}

  template <typename... Args>
  static scoped_refptr<DoTRequest> Create(Args&&... args) {
    return gurl_base::MakeRefCounted<DoTRequest>(std::forward<Args>(args)...);
  }
  ~DoTRequest();

  void close();

  using AsyncResolveCallback = absl::AnyInvocable<void(asio::error_code ec, struct addrinfo* addrinfo)>;
  void DoRequest(dns_message::DNStype dns_type, const std::string& host, int port, AsyncResolveCallback cb);

 private:
  // tcp connect
  // ssl handshake
  // generate HTTP request
  // send HTTP request
  // receive HTTP response
  // parse HTTP body

  void OnSocketConnect();
  void OnSSLConnect();
  void OnSSLWritable(asio::error_code ec);
  void OnSSLReadable(asio::error_code ec);
  void OnReadHeader();
  void OnReadBody();
  void OnParseDnsResponse();
  void OnDoneRequest(asio::error_code ec, struct addrinfo* addrinfo);

 private:
  enum ReadState {
    Read_Header,
    Read_Body,
  };
  ReadState read_state_ = Read_Header;
  size_t body_length_ = 0;

 private:
  asio::io_context& io_context_;
  asio::ip::tcp::socket socket_;
  asio::ip::tcp::endpoint endpoint_;

  const std::string& dot_host_;
  const int dot_port_;
  scoped_refptr<SSLSocket> ssl_socket_;
  const int ssl_socket_data_index_;
  SSL_CTX* ssl_ctx_;

  bool closed_ = false;
  dns_message::DNStype dns_type_;
  std::string host_;
  int port_;
  std::string service_;
  AsyncResolveCallback cb_;
  std::shared_ptr<IOBuf> buf_;
  std::shared_ptr<IOBuf> recv_buf_;
};

}  // namespace net

#endif  // H_NET_DOT_REQUEST_HPP
